import asyncio
import datetime as dt
import json
from pathlib import Path
from typing import Any, Dict, List, NamedTuple

import httpx
from dateutil.parser import parse
from httpx import HTTPError
from rich.console import Console
from rich.table import Table

console = Console()


class BuildRow(NamedTuple):
    team: str
    used: int
    total: int
    start_date: dt.datetime
    end_date: dt.datetime

    @property
    def percent_used(self) -> float:
        return 100 * self.used / self.total

    @property
    def percent_elapsed(self) -> float:
        return (
            100
            * (dt.datetime.now(tz=self.start_date.tzinfo) - self.start_date)
            / (self.end_date - self.start_date)
        )


def main():
    asyncio.run(run_async())


async def run_async():
    team_tokens = read_config()
    async with httpx.AsyncClient(timeout=3) as client:
        tasks = [
            make_request(client, team, token) for team, token in team_tokens.items()
        ]
        team_responses = await asyncio.gather(*tasks)
        rows = [
            parse_response(team, response_data)
            for team, response_data in team_responses
            if response_data
        ]
        print_table(rows)


def read_config():
    config_file = Path.home() / ".netlify-builds.json"
    return json.loads(config_file.read_text())


async def make_request(client, team, token):
    try:
        response = await client.get(
            f"https://api.netlify.com/api/v1/{team}/builds/status",
            headers={"content-length": "0", "authorization": f"Bearer {token}"},
        )
        response.raise_for_status()
    except HTTPError:
        return team, None
    return team, response.json()


def parse_response(team: str, response_data: Dict[str, Any]) -> BuildRow:
    minutes = response_data["minutes"]
    start_date = parse(minutes["period_start_date"])
    end_date = parse(minutes["period_end_date"])
    used = minutes["current"]
    total = minutes["included_minutes"]
    return BuildRow(
        team=team,
        used=used,
        total=total,
        start_date=start_date,
        end_date=end_date,
    )


def print_table(rows: List[BuildRow]):
    if not rows:
        console.print("No rows to print", style="bold red")
        return

    table = Table(show_header=True, header_style="bold blue")
    table.add_column("Team", style="bold yellow")
    table.add_column("Mins", style="dim", justify="right")
    table.add_column("Start Date", style="dim")
    table.add_column("End Date", style="dim")
    table.add_column("Elapsed", justify="right")
    table.add_column("Used", justify="right")

    for build_row in rows:
        style = "red" if build_row.percent_used > build_row.percent_elapsed else "green"
        table.add_row(
            build_row.team,
            f"{build_row.used} mins",
            f"{build_row.start_date:%Y-%m-%d}",
            f"{build_row.end_date:%Y-%m-%d}",
            f"[{style}]{build_row.percent_elapsed:.1f}%[/{style}]",
            f"[{style}]{build_row.percent_used:.1f}%[/{style}]",
        )

    console.print(table)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['netlify_builds']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.20.0,<0.21.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'rich>=10.0.0,<11.0.0']

entry_points = \
{'console_scripts': ['netlify-builds = netlify_builds.cli:main']}

setup_kwargs = {
    'name': 'netlify-builds',
    'version': '0.2.1',
    'description': 'A command line utility to check build usage across multiple Netlify accounts',
    'long_description': '# Netlify Builds\n\n<p align="center">\n  <a href="https://github.com/browniebroke/netlify-builds/actions?query=workflow%3ACI">\n    <img src="https://img.shields.io/github/workflow/status/browniebroke/netlify-builds/CI/main?label=CI&logo=github&style=flat-square" alt="CI Status" >\n  </a>\n  <a href="https://codecov.io/gh/browniebroke/netlify-builds">\n    <img src="https://img.shields.io/codecov/c/github/browniebroke/netlify-builds.svg?logo=codecov&logoColor=fff&style=flat-square" alt="Test coverage percentage">\n  </a>\n</p>\n<p align="center">\n  <a href="https://python-poetry.org/">\n    <img src="https://img.shields.io/badge/packaging-poetry-299bd7?style=flat-square&logo=data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA4AAAASCAYAAABrXO8xAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAJJSURBVHgBfZLPa1NBEMe/s7tNXoxW1KJQKaUHkXhQvHgW6UHQQ09CBS/6V3hKc/AP8CqCrUcpmop3Cx48eDB4yEECjVQrlZb80CRN8t6OM/teagVxYZi38+Yz853dJbzoMV3MM8cJUcLMSUKIE8AzQ2PieZzFxEJOHMOgMQQ+dUgSAckNXhapU/NMhDSWLs1B24A8sO1xrN4NECkcAC9ASkiIJc6k5TRiUDPhnyMMdhKc+Zx19l6SgyeW76BEONY9exVQMzKExGKwwPsCzza7KGSSWRWEQhyEaDXp6ZHEr416ygbiKYOd7TEWvvcQIeusHYMJGhTwF9y7sGnSwaWyFAiyoxzqW0PM/RjghPxF2pWReAowTEXnDh0xgcLs8l2YQmOrj3N7ByiqEoH0cARs4u78WgAVkoEDIDoOi3AkcLOHU60RIg5wC4ZuTC7FaHKQm8Hq1fQuSOBvX/sodmNJSB5geaF5CPIkUeecdMxieoRO5jz9bheL6/tXjrwCyX/UYBUcjCaWHljx1xiX6z9xEjkYAzbGVnB8pvLmyXm9ep+W8CmsSHQQY77Zx1zboxAV0w7ybMhQmfqdmmw3nEp1I0Z+FGO6M8LZdoyZnuzzBdjISicKRnpxzI9fPb+0oYXsNdyi+d3h9bm9MWYHFtPeIZfLwzmFDKy1ai3p+PDls1Llz4yyFpferxjnyjJDSEy9CaCx5m2cJPerq6Xm34eTrZt3PqxYO1XOwDYZrFlH1fWnpU38Y9HRze3lj0vOujZcXKuuXm3jP+s3KbZVra7y2EAAAAAASUVORK5CYII=" alt="Poetry">\n  </a>\n  <a href="https://github.com/ambv/black">\n    <img src="https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square" alt="black">\n  </a>\n  <a href="https://github.com/pre-commit/pre-commit">\n    <img src="https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white&style=flat-square" alt="pre-commit">\n  </a>\n</p>\n<p align="center">\n  <a href="https://pypi.org/project/netlify-builds/">\n    <img src="https://img.shields.io/pypi/v/netlify-builds.svg?logo=python&logoColor=fff&style=flat-square" alt="PyPI Version">\n  </a>\n  <img src="https://img.shields.io/pypi/pyversions/netlify-builds.svg?style=flat-square&logo=python&amp;logoColor=fff" alt="Supported Python versions">\n  <img src="https://img.shields.io/pypi/l/netlify-builds.svg?style=flat-square" alt="License">\n</p>\n\n\nA command line utility to check build usage across multiple Netlify accounts\n\n## Installation\n\nRecommended to install this via [pipx]:\n\n`pipx install netlify-builds`\n\n## Setup\n\nCreate a `.netlify-builds.json` in your home directory with the following shape:\n\n```json\n{\n  "team-name-1": "access-token-1",\n  "team-name-2": "access-token-2",\n  ...\n}\n```\n\nTo obtain the token for each team, open a private browsing session and login to your team dashboard and copy it from the local storage, it should be located under the key `nf-session`.\n\nDO NOT LOG OUT. Instead, simply close the private browsing session. If you log out, the token will be invalidated.\n\n## Profit\n\nYou\'re good to go, check all your accounts from the comfort of your terminal:\n\n```\n➜ netlify-builds\n┏━━━━━━━━━━━━━━━━━━━┳━━━━━━━━━━┳━━━━━━━━━━━━┳━━━━━━━━━━━━┳━━━━━━━━━┳━━━━━━━┓\n┃ Team              ┃     Mins ┃ Start Date ┃ End Date   ┃ Elapsed ┃  Used ┃\n┡━━━━━━━━━━━━━━━━━━━╇━━━━━━━━━━╇━━━━━━━━━━━━╇━━━━━━━━━━━━╇━━━━━━━━━╇━━━━━━━┩\n│ team-blue         │   5 mins │ 2020-11-16 │ 2020-12-16 │   11.6% │  1.7% │\n│ team-red          │ 182 mins │ 2020-10-27 │ 2020-11-27 │   75.9% │ 60.7% │\n│ team-green        │  46 mins │ 2020-11-02 │ 2020-12-02 │   58.3% │ 15.3% │\n└───────────────────┴──────────┴────────────┴────────────┴─────────┴───────┘\n```\n\nIf you\'re likely to exceed the free quota (300 mins) the rows will appear in red, otherwise in green.\n\n## Contributors ✨\n\nThanks goes to these wonderful people ([emoji key]):\n\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable -->\n<table>\n  <tr>\n    <td align="center"><a href="https://browniebroke.com/"><img src="https://avatars1.githubusercontent.com/u/861044?v=4?s=80" width="80px;" alt=""/><br /><sub><b>Bruno Alla</b></sub></a><br /><a href="https://github.com/browniebroke/netlify-builds/commits?author=browniebroke" title="Code">💻</a> <a href="https://github.com/browniebroke/netlify-builds/commits?author=browniebroke" title="Documentation">📖</a> <a href="#ideas-browniebroke" title="Ideas, Planning, & Feedback">🤔</a></td>\n  </tr>\n</table>\n\n<!-- markdownlint-restore -->\n<!-- prettier-ignore-end -->\n\n<!-- ALL-CONTRIBUTORS-LIST:END -->\n\nThis project follows the [all-contributors] specification. Contributions of any kind welcome!\n\n## Credits\n\nThis package was created with [Cookiecutter] and the [browniebroke/cookiecutter-pypackage][bb-cc-pypkg] project template.\n\n[pipx]: https://pipxproject.github.io/pipx/\n[emoji key]: https://allcontributors.org/docs/en/emoji-key\n[all-contributors]: https://github.com/all-contributors/all-contributors\n[cookiecutter]: https://github.com/audreyr/cookiecutter\n[bb-cc-pypkg]: https://github.com/browniebroke/cookiecutter-pypackage\n',
    'author': 'Bruno Alla',
    'author_email': 'alla.brunoo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/browniebroke/netlify-builds',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
